apc.indiv.design.collinear <- function(data, unit=1, 
                                       n.coh.excl.start=0, n.coh.excl.end=0,
                                       n.per.excl.start=0, n.per.excl.end=0,
                                       n.age.excl.start=0, n.age.excl.end=0){
  # ---------------------------------------------------------------------------  
  # This function generates a collinear design matrix which encompasses all
  # possible APC submodels and covariate combinations.
  # ---------------------------------------------------------------------------
  
  ## Step 1: Checks ####
  
  # check data has appropriate variable names
  if(isTRUE(("age" %in% colnames(data)) 
            && ("period" %in% colnames(data))
            && ("cohort" %in% colnames(data))) == FALSE)
    stop("apc.error: data missing one of age, period, cohort")
  
  # add cell.name
  if (!"cell.name" %in% colnames(data)) {
    cell.name <- paste("ik", as.character(data$age), as.character(data$cohort),
                       sep="_")
    data <- cbind(cell.name, data)
  } else {
    warning("Variable cell.name must be unique identifier of age-cohort cells; 
            if preexisting variable cell.name has alternate meaning please 
            rename")
  }
  
  ## Step 2: Get data dimensions and mu.index ####
  
  # get data minimum values 
  # first two used in construction of i.value, k.value
  age1 <- min(data$age   )
  coh1 <- min(data$cohort)
  # third used only in naming later
  per1 <- min(data$period)
  
  # use minimum values to construct i, j, and k
  i.value <- (data$age - age1)/unit + 1 + n.age.excl.start
  k.value <- (data$cohort - coh1)/unit + 1 + n.coh.excl.start
  j.value <- i.value + k.value - 1 + n.per.excl.start
  
  # get unique combinations of i, j, and k; i.e. list of ik cells
  data.wijk <- cbind(data, i.value, j.value, k.value)
  mu.index <- plyr::count(data.wijk, c("i.value", "j.value", "k.value"))
  mu.index <- mu.index[mu.index$freq>0, ]
  mu.index <- mu.index[, !names(mu.index)=="freq"]
  
  # define L and U: anchor points of plane
  L <- min(j.value) - 1     # correct bc this is the minimum observed j
  U <- as.integer((L+3)/2)
  
  # stop if something has gone wrong here
  stopifnot(exists("L")==TRUE)  
  stopifnot(exists("U")==TRUE)
  
  # get dimensionality of data
  I <- as.numeric(length(unique(data.wijk$i.value)))
  J <- as.numeric(length(unique(data.wijk$j.value)))
  K <- as.numeric(length(unique(data.wijk$k.value)))
  
  ## Step 3: create and fill design matrix - one row per ik cell ####
  
  #create space of design matrix
  n.rows <- nrow(mu.index)
  n.param.design <- I+J+K-2 # -2 rather than -3 because we allow for a period
  # slope; in the submodels with P and tP, it's easier to estimate a period 
  # slope than an equality constraint
  design <- matrix(data=0, nrow=n.rows, ncol=n.param.design+3)
  # the extra three are for index: i, j, k
  
  #begin by defining L.odd
  L.odd <- !L %% 2 == 0 
  
  #fill row-by-row
  for (row in 1:n.rows) {
    i <- mu.index[row, 1]
    j <- mu.index[row, 2]
    k <- mu.index[row, 3]
    design[row, 1] <- 1 # the intercept
    design[row, 2] <- i - U # the first (age) slope
    design[row, 4] <- k - U # the second (cohort) slope
    design[row, 3] <- design[row, 2] + design[row, 4] # period slope
    
    #age DDs
    # note 4 plane pieces => start at 4+1
    if (i < U) 
      design[row, (4 + i - n.age.excl.start):
               (4 + U - 1 - n.age.excl.start)] <- seq(1, U - i)
    # backward cumulation from DD3 to DD(U-1)
    if (i > U + 1) 
      design[row, (4 + U - n.age.excl.start):
               (4 + i - 2 - n.age.excl.start)] <- seq(i - U - 1, 1)
    # forward cumulation from DDU to DDI
    # note DDI is only the (I-2)nd column of DDs b/c start at DD3
    
    #period DDs
    # note 4 plane pieces + I-2 age DDs => start at 2+I+1
    # CASE 1: L odd
    if (L.odd && j == (L + 1))
      design[row, (2 + I + 1)] <- 1
    # single backward cumulation of DD(L+3)
    if (L.odd && j > (L + 3))
      design[row, (2 + I + 1 + L.odd):(2 + I + L.odd + j - (L + 3))] <-
      seq(j - (L + 3), 1)
    # forward cumulation from DD(L+4) to DD(L+J)
    # note DD(L+J) is the (J-3)rd column of DDs here b/c start at DD(L+4);
    # recall DD(L+3) already covered with backward cumulation
    # CASE 2: L even
    if ((!L.odd) && j > (L + 2))
      design[row, (2 + I + 1):(2 + I + j - (L + 2))] <- seq(j - (L + 2), 1)
    # forward cumulation from DD(L+3) to DD(L+J)
    # DD(L+J) is the (J-2)nd column of DDs here b/c start at DD(L+3)
    
    #cohort DDs
    # note 4 plane pieces + I-2 age + J-2 period DDs => start at I+J+1
    # also note symmetry to age DDs
    if (k < U) 
      design[row, (I + J + k - n.coh.excl.start):
               (I + J + U - 1 - n.coh.excl.start)] <- seq(1, U - k)
    # backward cumulation from DD3 to DD(U-1)
    if (k > U + 1) 
      design[row, (I + J + U - n.coh.excl.start):
               (I + J + k - 2 - n.coh.excl.start)] <- seq(k - U - 1, 1)
    # forward cumulation from DDU to DDI
    # note DDI is only the (I-2)nd column of DDs b/c start at DD3
    
    #index
    design[row, n.param.design+1] <- i
    design[row, n.param.design+2] <- j
    design[row, n.param.design+3] <- k
  }
  
  # convert matrix -> data frame
  designdf <- as.data.frame(design)
  
  # assign names to variables in design dataframe
  firstfour <- c("level", "age slope", "period slope", "cohort slope")
  age.names <- paste("DD_age", seq(age1+2*unit, age1+2*unit+(I-2)*unit-1, 
                                   unit), sep="_")
  per.names <- paste("DD_period", seq(per1+2*unit, per1+2*unit+(J-2)*unit-1, 
                                      unit), sep="_")
  coh.names <- paste("DD_cohort", seq(coh1+2*unit, coh1+2*unit+(K-2)*unit-1, 
                                      unit), sep="_")
  index <- c("i.value", "j.value", "k.value")
  colnames(designdf) <- c(firstfour, age.names, per.names, coh.names, index)
  
  ## Step 4: join design matrix with full data using ijk index values ####
  
  # associates APC design matrix with each individual datapoint as needed
  data.with.design <- join(data.wijk, designdf, by=c("i.value", "j.value", 
                                                     "k.value"))
  
  # drop information that is redundant in future commands
  exclude.all <- c("X", "indiv.ID", "age", "period", "cohort", "i.value", 
                   "j.value", "k.value")
  first.exclusion <- names(data.with.design) %in% c(exclude.all)
  full.design.collinear <- data.with.design[!first.exclusion]
  
  # check join was successful
  if (anyNA(full.design.collinear)) 
    stop("NA in design matrix, check data or n.excl age/per/coh")
  
  # ---------------------------------------------------------------------------
  # Return valuables
  valuables <- list(structure.design.collinear = designdf, 
                    full.design.collinear = full.design.collinear, 
                    unit = unit,
                    age1 = age1,
                    per1 = per1,
                    coh1 = coh1,
                    age.max = I,
                    per.max = J,
                    coh.max = K,
                    per.zero = L,
                    per.odd = L.odd,
                    U = U)
  
  return (valuables)
  }