apc.indiv.ftable <- function(data, dep.var, covariates=NULL, unit=1, 
                             n.coh.excl.start=0, n.coh.excl.end=0, 
                             n.age.excl.start=0, n.age.excl.end=0, 
                             n.per.excl.start=0, n.per.excl.end=0){
  ######################	
  # This function generates a table comparing all APC submodels to the 
  # TS model and the full APC model in terms of an F-test of fit.
  ######################	
  # Step 1: write function that uses output from an F-test against the 
  #TS and an F-test against the full
  # APC model to generate a line suitable for the final table.
  get.table.line <- function(ftest1, ftest2, first=FALSE, second=FALSE){
    if(isTRUE(first)) # first line is TS model so aic only
      line <- c(NA, NA, NA, NA, NA, NA, round(ftest1$aic.big, 3), 
                round(ftest1$lik.big, 3)) 
    else if(isTRUE(second)) 
      # second line is full APC model so not compared to itself
      line <- c(round(ftest1$fstat, 3), ftest1$df.num, round(ftest1$p.value,3),
                NA,
                NA, NA, round(ftest1$aic.small,3), round(ftest1$lik.small, 3))
    else
      line <- c(round(ftest1$fstat, 3), ftest1$df.num, round(ftest1$p.value,3), 
                round(ftest2$fstat, 3),
                ftest2$df.num, round(ftest2$p.value, 3), round(ftest2$aic.small,3), 
                round(ftest2$lik.small, 3))
    return(line)
  }
  ######################	
  # Step 2: Other setup
  
  # List of submodels to be investigated
  model.design.list	<- c("TS", "APC","AP","AC","PC","Ad","Pd","Cd","A","P",
                         "C","t","tA","tP","tC", "1")
  # Empty table
  fit.tab <- matrix(nrow=length(model.design.list),ncol=8,data=NA)
  # Full APC model for comparison
  fullAPCcollinear <- apc.indiv.design.collinear(data, 
                                                 unit=unit,
                                                 n.coh.excl.start=n.coh.excl.start,
                                                 n.coh.excl.end=n.coh.excl.end,
                                                 n.age.excl.start = n.age.excl.start,
                                                 n.age.excl.end = n.age.excl.end,
                                                 n.per.excl.start = n.per.excl.start,
                                                 n.per.excl.end = n.per.excl.end)
  fullAPCdesign <- apc.indiv.design.model(fullAPCcollinear, dep.var = dep.var, 
                                          covariates = covariates)
  fullAPCfit <- apc.indiv.fit.model(fullAPCdesign, model.family="gaussian")
  # TS model for comparison
  TSfit <- apc.indiv.estimate.TS(data, dep.var=dep.var, covariates=covariates)
  ######################	
  # Step 3: Generate table
  
  # First line: TS model
  ftest.full.vs.TS <- apc.indiv.ftest.TS(existing.small.model.fit = 
                                           fullAPCfit, existing.big.model.fit = TSfit)
  first.line <- get.table.line(ftest1 = ftest.full.vs.TS, first=TRUE)
  fit.tab[1, 1:8] <- first.line
  # Second line: full APC model
  second.line <- get.table.line(ftest1 = ftest.full.vs.TS, second=TRUE)
  fit.tab[2, 1:8] <- second.line
  # Remaining submodels
  for (i in 3:length(model.design.list)){
    againstTS <- apc.indiv.ftest.TS(data,  
                                    small.model = model.design.list[i],
                                    dep.var = dep.var,covariates = covariates, 
                                    unit = unit,
                                    n.coh.excl.start = n.coh.excl.start, 
                                    n.coh.excl.end = n.coh.excl.end,
                                    n.age.excl.start = n.age.excl.start,
                                    n.age.excl.end = n.age.excl.end,
                                    n.per.excl.start = n.per.excl.start,
                                    n.per.excl.end = n.per.excl.end,
                                    existing.big.model.fit = TSfit, 
                                    existing.collinear = fullAPCcollinear)
    againstAPC <- apc.indiv.ftest.fullapc(data, 
                                          small.model = model.design.list[i],
                                          dep.var=dep.var, covariates=covariates, 
                                          unit = unit, 
                                          n.coh.excl.start = n.coh.excl.start, 
                                          n.coh.excl.end = n.coh.excl.end,
                                          n.age.excl.start = n.age.excl.start,
                                          n.age.excl.end = n.age.excl.end,
                                          n.per.excl.start = n.per.excl.start,
                                          n.per.excl.end = n.per.excl.end,
                                          existing.big.model.fit = fullAPCfit, 
                                          existing.collinear = fullAPCcollinear)
    fit.tab[i, 1:8] <- get.table.line(againstTS, againstAPC)
  }
  # Change table column and row names 
  df.againstTS <- paste("DF ( * , ", as.character(againstTS$df.denom), ")", 
                        sep="")
  df.againstAPC <- paste("DF ( * , ", as.character(againstAPC$df.denom), ")",
                         sep="")
  
  table.colnames <- c("F-test vs TS", df.againstTS, "p-value", "F-test vs APC", 
                      df.againstAPC, "p-value", "AIC", "lik")
  
  colnames(fit.tab) <- table.colnames
  rownames(fit.tab) <- model.design.list
  ######################	
  valuables <- list(table = fit.tab,
                    NR.report = NULL)
  return(valuables)
}